@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
 """
-===========================================================================
 This file is part of py-eddy-tracker.
 
     py-eddy-tracker is free software: you can redistribute it and/or modify
@@ -16,16 +15,21 @@ This file is part of py-eddy-tracker.
     You should have received a copy of the GNU General Public License
     along with py-eddy-tracker.  If not, see <http://www.gnu.org/licenses/>.
 
-Copyright (c) 2014-2020 by Evan Mason
+Copyright (c) 2014-2020 by Evan Mason and Antoine Delepoulle
 Email: evanmason@gmail.com
-===========================================================================
+
 """
 
-from argparse import ArgumentParser
 import logging
-import numpy
+from argparse import ArgumentParser
+
 import zarr
 
+from ._version import get_versions
+
+__version__ = get_versions()["version"]
+del get_versions
+
 
 def start_logger():
     FORMAT_LOG = (
@@ -65,16 +69,14 @@ class ColoredFormatter(logging.Formatter):
 
 
 class EddyParser(ArgumentParser):
-    """General parser for applications
-    """
+    """General parser for applications"""
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.add_base_argument()
 
     def add_base_argument(self):
-        """Base arguments
-        """
+        """Base arguments"""
         self.add_argument(
             "-v",
             "--verbose",
@@ -140,6 +142,15 @@ VAR_DESCR = dict(
         nc_dims=("obs",),
         nc_attr=dict(),
     ),
+    distance_next=dict(
+        attr_name=None,
+        nc_name="distance_next",
+        nc_type="float32",
+        output_type="uint16",
+        scale_factor=50.0,
+        nc_dims=("obs",),
+        nc_attr=dict(long_name="Distance to next position", units="m"),
+    ),
     virtual=dict(
         attr_name=None,
         nc_name="observation_flag",
@@ -235,6 +246,28 @@ VAR_DESCR = dict(
             "the eddy and the SSH around the effective contour defining the eddy edge",
         ),
     ),
+    speed_area=dict(
+        attr_name="speed_area",
+        nc_name="speed_area",
+        nc_type="float32",
+        nc_dims=("obs",),
+        nc_attr=dict(
+            long_name="Speed area",
+            units="m^2",
+            comment="Area enclosed by speed contour in m^2",
+        ),
+    ),
+    effective_area=dict(
+        attr_name="effective_area",
+        nc_name="effective_area",
+        nc_type="float32",
+        nc_dims=("obs",),
+        nc_attr=dict(
+            long_name="Effective area",
+            units="m^2",
+            comment="Area enclosed by effective contour in m^2",
+        ),
+    ),
     speed_average=dict(
         attr_name="speed_average",
         scale_factor=0.0001,
@@ -268,18 +301,14 @@ VAR_DESCR = dict(
         nc_name="i",
         nc_type="uint16",
         nc_dims=("obs",),
-        nc_attr=dict(
-            long_name="Longitude index in the grid of the detection",
-        ),
+        nc_attr=dict(long_name="Longitude index in the grid of the detection"),
     ),
     j=dict(
         attr_name="j",
         nc_name="j",
         nc_type="uint16",
         nc_dims=("obs",),
-        nc_attr=dict(
-            long_name="Latitude index in the grid of the detection",
-        ),
+        nc_attr=dict(long_name="Latitude index in the grid of the detection"),
     ),
     eke=dict(
         attr_name="eke",
@@ -317,8 +346,8 @@ VAR_DESCR = dict(
         nc_attr=dict(
             long_name="Speed Radius",
             units="m",
-            comment="Radius of a circle whose area is equal to that \
-                enclosed by the contour of maximum circum-average speed",
+            comment="Radius of a circle whose area is equal to that "
+            "enclosed by the contour of maximum circum-average speed",
         ),
     ),
     track=dict(
@@ -328,8 +357,7 @@ VAR_DESCR = dict(
         nc_type="uint32",
         nc_dims=("obs",),
         nc_attr=dict(
-            long_name="Trajectory number",
-            comment="Trajectory identification number",
+            long_name="Trajectory number", comment="Trajectory identification number"
         ),
     ),
     sub_track=dict(
@@ -338,8 +366,7 @@ VAR_DESCR = dict(
         nc_type="uint32",
         nc_dims=("obs",),
         nc_attr=dict(
-            long_name="Segment Number",
-            comment="Segment number inside a group",
+            long_name="Segment Number", comment="Segment number inside a group"
         ),
     ),
     n=dict(
@@ -360,8 +387,8 @@ VAR_DESCR = dict(
         nc_type="f4",
         filters=[zarr.Delta("i2")],
         output_type="i2",
-        scale_factor=numpy.float32(0.01),
-        add_offset=180,
+        scale_factor=0.01,
+        add_offset=180.0,
         nc_dims=("obs", "NbSample"),
         nc_attr=dict(
             long_name="Effective Contour Longitudes",
@@ -377,7 +404,7 @@ VAR_DESCR = dict(
         nc_type="f4",
         filters=[zarr.Delta("i2")],
         output_type="i2",
-        scale_factor=numpy.float32(0.01),
+        scale_factor=0.01,
         nc_dims=("obs", "NbSample"),
         nc_attr=dict(
             long_name="Effective Contour Latitudes",
@@ -404,8 +431,8 @@ VAR_DESCR = dict(
         nc_type="f4",
         filters=[zarr.Delta("i2")],
         output_type="i2",
-        scale_factor=numpy.float32(0.01),
-        add_offset=180,
+        scale_factor=0.01,
+        add_offset=180.0,
         nc_dims=("obs", "NbSample"),
         nc_attr=dict(
             long_name="Speed Contour Longitudes",
@@ -421,7 +448,7 @@ VAR_DESCR = dict(
         nc_type="f4",
         filters=[zarr.Delta("i2")],
         output_type="i2",
-        scale_factor=numpy.float32(0.01),
+        scale_factor=0.01,
         nc_dims=("obs", "NbSample"),
         nc_attr=dict(
             long_name="Speed Contour Latitudes",
@@ -462,7 +489,7 @@ VAR_DESCR = dict(
         output_type="u1",
         scale_factor=0.4,
         nc_dims=("obs",),
-        nc_attr=dict(units="%", comment="score", long_name="Score",),
+        nc_attr=dict(units="%", comment="score", long_name="Score"),
     ),
     index_other=dict(
         attr_name=None,
@@ -531,7 +558,7 @@ VAR_DESCR = dict(
         old_nc_name=["Chl"],
         nc_type="f4",
         nc_dims=("obs",),
-        nc_attr=dict(long_name="Log base 10 chlorophyll", units="Log(Chl/[mg/m^3])",),
+        nc_attr=dict(long_name="Log base 10 chlorophyll", units="Log(Chl/[mg/m^3])"),
     ),
     dchl=dict(
         attr_name=None,
@@ -551,7 +578,8 @@ VAR_DESCR = dict(
         nc_type="f4",
         nc_dims=("obs",),
         nc_attr=dict(
-            long_name="Log base 10 background chlorophyll", units="Log(Chl/[mg/m^3])",
+            long_name="Log base 10 background chlorophyll",
+            units="Log(Chl/[mg/m^3])",
         ),
     ),
     year=dict(
@@ -560,7 +588,7 @@ VAR_DESCR = dict(
         old_nc_name=["Year"],
         nc_type="u2",
         nc_dims=("obs",),
-        nc_attr=dict(long_name="Year", units="year",),
+        nc_attr=dict(long_name="Year", units="year"),
     ),
     month=dict(
         attr_name=None,
@@ -568,7 +596,7 @@ VAR_DESCR = dict(
         old_nc_name=["Month"],
         nc_type="u1",
         nc_dims=("obs",),
-        nc_attr=dict(long_name="Month", units="month",),
+        nc_attr=dict(long_name="Month", units="month"),
     ),
     day=dict(
         attr_name=None,
@@ -576,7 +604,7 @@ VAR_DESCR = dict(
         old_nc_name=["Day"],
         nc_type="u1",
         nc_dims=("obs",),
-        nc_attr=dict(long_name="Day", units="day",),
+        nc_attr=dict(long_name="Day", units="day"),
     ),
     nb_contour_selected=dict(
         attr_name=None,
@@ -600,8 +628,3 @@ for key in VAR_DESCR.keys():
     VAR_DESCR_inv[VAR_DESCR[key]["nc_name"]] = key
     for key_old in VAR_DESCR[key].get("old_nc_name", list()):
         VAR_DESCR_inv[key_old] = key
-
-from ._version import get_versions
-
-__version__ = get_versions()["version"]
-del get_versions
