@@ -1,66 +1,74 @@
 # -*- coding: utf-8 -*-
 """
+Class to load and manipulate RegularGrid and UnRegularGrid
 """
 import logging
+from datetime import datetime
+
+from cv2 import filter2D
+from matplotlib.path import Path as BasePath
+from netCDF4 import Dataset
+from numba import njit
+from numba import types as numba_types
 from numpy import (
-    concatenate,
-    empty,
-    where,
-    array,
-    sin,
-    deg2rad,
-    pi,
-    ones,
-    cos,
-    ma,
-    int8,
-    histogram2d,
     arange,
-    float_,
-    linspace,
+    array,
+    ceil,
+    concatenate,
+    cos,
+    deg2rad,
+    empty,
     errstate,
+    exp,
+    float_,
+    histogram2d,
+    int8,
     int_,
     interp,
+    isnan,
+    linspace,
+    ma,
+)
+from numpy import mean as np_mean
+from numpy import (
     meshgrid,
     nan,
-    ceil,
-    sinc,
-    isnan,
-    percentile,
-    zeros,
-    round_,
     nanmean,
-    exp,
-    mean as np_mean,
+    ones,
+    percentile,
+    pi,
+    round_,
+    sin,
+    sinc,
+    where,
+    zeros,
 )
-from datetime import datetime
-from scipy.special import j1
-from netCDF4 import Dataset
-from scipy.ndimage import gaussian_filter, convolve
-from scipy.interpolate import RectBivariateSpline, interp1d
-from scipy.spatial import cKDTree
-from scipy.signal import welch
-from cv2 import filter2D
-from numba import njit, types as numba_types
-from matplotlib.path import Path as BasePath
 from pint import UnitRegistry
-from ..observations.observation import EddiesObservations
-from ..eddy_feature import Amplitude, Contours
+from scipy.interpolate import RectBivariateSpline, interp1d
+from scipy.ndimage import convolve, gaussian_filter
+from scipy.signal import welch
+from scipy.spatial import cKDTree
+from scipy.special import j1
+
 from .. import VAR_DESCR
+from ..eddy_feature import Amplitude, Contours
 from ..generic import (
+    bbox_indice_regular,
+    coordinates_to_local,
     distance,
     interp2d_geo,
-    fit_circle,
-    uniform_resample,
-    coordinates_to_local,
     local_to_coordinates,
+    nearest_grd_indice,
+    uniform_resample,
 )
+from ..observations.observation import EddiesObservations
 from ..poly import (
-    poly_contain_poly,
-    winding_number_grid_in_poly,
-    winding_number_poly,
     create_vertice,
+    fit_circle,
+    get_pixel_in_regular,
     poly_area,
+    poly_contain_poly,
+    winding_number_poly,
 )
 
 logger = logging.getLogger("pet")
@@ -151,7 +159,7 @@ def _circle_from_equal_area(vertice):
         # logger.debug('%d coordinates %s,%s', len(lons),lons,
         # lats)
         return 0, -90, nan, nan
-    return lon0, lat0, (poly_area(create_vertice(c_x, c_y)) / pi) ** 0.5, nan
+    return lon0, lat0, (poly_area(c_x, c_y) / pi) ** 0.5, nan
 
 
 @njit(cache=True, fastmath=True)
@@ -174,36 +182,6 @@ def _fit_circle_path(vertice):
     return centlon, centlat, eddy_radius, err
 
 
-@njit(cache=True, fastmath=True)
-def _get_pixel_in_regular(vertices, x_c, y_c, x_start, x_stop, y_start, y_stop):
-    if x_stop < x_start:
-        x_ref = vertices[0, 0]
-        x_array = (
-            (concatenate((x_c[x_start:], x_c[:x_stop])) - x_ref + 180) % 360
-            + x_ref
-            - 180
-        )
-        return winding_number_grid_in_poly(
-            x_array,
-            y_c[y_start:y_stop],
-            x_start,
-            x_stop,
-            x_c.shape[0],
-            y_start,
-            vertices,
-        )
-    else:
-        return winding_number_grid_in_poly(
-            x_c[x_start:x_stop],
-            y_c[y_start:y_stop],
-            x_start,
-            x_stop,
-            x_c.shape[0],
-            y_start,
-            vertices,
-        )
-
-
 @njit(cache=True, fastmath=True)
 def _get_pixel_in_unregular(vertices, x_c, y_c, x_start, x_stop, y_start, y_stop):
     nb_x, nb_y = x_stop - x_start, y_stop - y_start
@@ -323,7 +301,7 @@ class GridDataset(object):
         self.interpolators = dict()
         if centered is None:
             logger.warning(
-                "We assume pixel position of grid is center for %s", filename,
+                "We assume pixel position of grid is center for %s", filename
             )
         if not unset:
             self.load_general_features()
@@ -331,8 +309,11 @@ class GridDataset(object):
 
     @property
     def is_centered(self):
-        """Give information if pixel is describe with center position or
+        """Give True if pixel is described with its center's position or
         a corner
+
+        :return: True if centered
+        :rtype: bool
         """
         if self.centered is None:
             return True
@@ -340,8 +321,7 @@ class GridDataset(object):
             return self.centered
 
     def load_general_features(self):
-        """Load attrs
-        """
+        """Load attrs to  be stored in object"""
         logger.debug(
             "Load general feature from %(filename)s", dict(filename=self.filename)
         )
@@ -351,7 +331,7 @@ class GridDataset(object):
             self.variables_description = dict()
             for i, v in h.variables.items():
                 args = (i, v.datatype)
-                kwargs = dict(dimensions=v.dimensions, zlib=True,)
+                kwargs = dict(dimensions=v.dimensions, zlib=True)
                 if hasattr(v, "_FillValue"):
                     kwargs["fill_value"] = (v._FillValue,)
                 attrs = dict()
@@ -367,7 +347,9 @@ class GridDataset(object):
             self.global_attrs = {attr: getattr(h, attr) for attr in h.ncattrs()}
 
     def write(self, filename):
-        """Write dataset output with same format like input
+        """Write dataset output with same format as input
+
+        :param str filename: filename used to save the grid
         """
         with Dataset(filename, "w") as h_out:
             for dimension, size in self.dimensions.items():
@@ -401,7 +383,9 @@ class GridDataset(object):
                 setattr(h_out, attr, value)
 
     def load(self):
-        """Load variable (data)
+        """
+        Load variable (data).
+        Get coordinates and setup coordinates function
         """
         x_name, y_name = self.coordinates
         with Dataset(self.filename) as h:
@@ -416,6 +400,20 @@ class GridDataset(object):
         self.setup_coordinates()
         self.init_pos_interpolator()
 
+    @staticmethod
+    def c_to_bounds(c):
+        """
+        Centred coordinates to bounds coordinates
+
+        :param array c: centred coordinates to translate
+        :return: bounds coordinates
+        """
+        bounds = concatenate((c, (2 * c[-1] - c[-2],)))
+        d = bounds[1:] - bounds[:-1]
+        bounds[:-1] -= d / 2
+        bounds[-1] -= d[-1] / 2
+        return bounds
+
     def setup_coordinates(self):
         x_name, y_name = self.coordinates
         if self.is_centered:
@@ -449,11 +447,11 @@ class GridDataset(object):
                 raise Exception("not write")
 
     def is_circular(self):
-        """Check grid circularity
-        """
+        """Check grid circularity"""
         return False
 
     def units(self, varname):
+        """Get unit from variable"""
         stored_units = self.variables_description[varname]["attrs"].get("units", None)
         if stored_units is not None:
             return stored_units
@@ -462,9 +460,13 @@ class GridDataset(object):
             if hasattr(var, "units"):
                 return var.units
 
+    @property
+    def variables(self):
+        return self.variables_description.keys()
+
     def copy(self, grid_in, grid_out):
         """
-        Duplicate a variable
+        Duplicate the variable from grid_in in grid_out
 
         :param grid_in:
         :param grid_out:
@@ -479,8 +481,24 @@ class GridDataset(object):
         )
         self.vars[grid_out] = self.grid(grid_in).copy()
 
+    def add_grid(self, varname, grid):
+        """
+        Add a grid in handler
+
+        :param str varname: name of the future grid
+        :param array grid: grid array
+        """
+        self.vars[varname] = grid
+
     def grid(self, varname, indexs=None):
-        """give grid required
+        """Give the grid required
+
+        :param str varname: Variable to get
+        :param dict,None indexs: If defined dict must have dimensions name as key
+        :return: array asked, reduced by the indexes
+        :rtype: array
+
+        .. minigallery:: py_eddy_tracker.GridDataset.grid
         """
         if indexs is None:
             indexs = dict()
@@ -517,8 +535,7 @@ class GridDataset(object):
         return self.vars[varname]
 
     def grid_tiles(self, varname, slice_x, slice_y):
-        """give grid tiles required, without buffer system
-        """
+        """Give the grid tiles required, without buffer system"""
         coordinates_dims = list(self.x_dim)
         coordinates_dims.extend(list(self.y_dim))
         logger.debug(
@@ -549,21 +566,26 @@ class GridDataset(object):
         return data
 
     def high_filter(self, grid_name, w_cut, **kwargs):
-        """create a high filter with a low one
+        """Return the grid high-pass filtered, by substracting to the grid the low-pass filter (default: order=1)
+
+        :param grid_name: the name of the grid
+        :param int, w_cut: the half-power wavelength cutoff (km)
         """
         result = self._low_filter(grid_name, w_cut, **kwargs)
         self.vars[grid_name] -= result
 
     def low_filter(self, grid_name, w_cut, **kwargs):
-        """low filtering
+        """Return the grid low-pass filtered (default: order=1)
+
+        :param grid_name: the name of the grid
+        :param int, w_cut: the half-power wavelength cutoff (km)
         """
         result = self._low_filter(grid_name, w_cut, **kwargs)
         self.vars[grid_name] -= self.vars[grid_name] - result
 
     @property
     def bounds(self):
-        """Give bound
-        """
+        """Give bounds"""
         return (
             self.x_bounds.min(),
             self.x_bounds.max(),
@@ -584,25 +606,29 @@ class GridDataset(object):
         precision=None,
         force_height_unit=None,
         force_speed_unit=None,
+        **kwargs,
     ):
         """
-        Compute eddy identification on specified grid
+        Compute eddy identification on the pecified grid
 
-        :param str grid_height: Grid name of height
+        :param str grid_height: Grid name of Sea Surface Height
         :param str uname: Grid name of u speed component
         :param str vname: Grid name of v speed component
-        :param datetime.datetime date: Date which will be store in object to date data
+        :param datetime.datetime date: Date which will be stored in object to date data
         :param float,int step: Height between two layers in m
-        :param float,int shape_error: Maximal error allow for outter contour in %
-        :param int sampling: Sampling of contour and speed profile
-        :param (int,int),None pixel_limit: Min and max of pixel which must be inside inner and outer contour to be considered like an eddy
-        :param float,None precision: Truncate value at the defined precision in m
-        :param str force_height_unit: Unit to used for height unit
-        :param str force_speed_unit: Unit to used for speed unit
+        :param float,int shape_error: Maximal error allowed for outter contour in %
+        :param int sampling: Number of points to store contours and speed profile
+        :param (int,int),None pixel_limit:
+            Min and max number of pixels inside the inner and the outer contour to be considered as an eddy
+        :param float,None precision: Truncate values at the defined precision in m
+        :param str force_height_unit: Unit used for height unit
+        :param str force_speed_unit: Unit used for speed unit
+        :param dict kwargs: Argument given to amplitude
 
         :return: Return a list of 2 elements: Anticyclone and Cyclone
         :rtype: py_eddy_tracker.observations.observation.EddiesObservations
 
+        .. minigallery:: py_eddy_tracker.GridDataset.eddy_identification
         """
         if not isinstance(date, datetime):
             raise Exception("Date argument be a datetime object")
@@ -630,7 +656,7 @@ class GridDataset(object):
             if precision is not None:
                 precision /= factor
 
-        # Get h grid
+        # Get ssh grid
         data = self.grid(grid_height).astype("f8")
         # In case of a reduce mask
         if len(data.mask.shape) == 0 and not data.mask:
@@ -681,7 +707,7 @@ class GridDataset(object):
             "contour_lat_s",
             "uavg_profile",
         ]
-        # Compute cyclonic and anticylonic research:
+        # Complete cyclonic and anticylonic research:
         a_and_c = list()
         for anticyclonic_search in [True, False]:
             eddies = list()
@@ -714,6 +740,7 @@ class GridDataset(object):
 
                     # Filter for shape
                     if aerr < 0 or aerr > shape_error or isnan(aerr):
+                        contour.reject = 1
                         continue
 
                     # Find all pixels in the contour
@@ -721,9 +748,11 @@ class GridDataset(object):
 
                     # Check if pixels in contour are masked
                     if has_masked_value(data.mask, i_x_in, i_y_in):
+                        if contour.reject == 0:
+                            contour.reject = 2
                         continue
 
-                    # Test to know cyclone or anticyclone
+                    # Test of the rotating sense: cyclone or anticyclone
                     if has_value(
                         data, i_x_in, i_y_in, cvalues, below=anticyclonic_search
                     ):
@@ -734,6 +763,7 @@ class GridDataset(object):
                         contour.nb_pixel < pixel_limit[0]
                         or contour.nb_pixel > pixel_limit[1]
                     ):
+                        contour.reject = 3
                         continue
 
                     # Compute amplitude
@@ -743,12 +773,13 @@ class GridDataset(object):
                         data,
                         anticyclonic_search=anticyclonic_search,
                         level=self.contours.levels[corrected_coll_index],
-                        step=step,
+                        interval=step,
+                        **kwargs,
                     )
                     # If we have a valid amplitude
                     if (not amp.within_amplitude_limits()) or (amp.amplitude == 0):
+                        contour.reject = 4
                         continue
-
                     if reset_centroid:
 
                         if self.is_circular():
@@ -764,7 +795,7 @@ class GridDataset(object):
                             centlon_e = x[centi, centj]
                             centlat_e = y[centi, centj]
 
-                    # centlat_e and centlon_e must be index of maximum, we will loose some inner contour, if it's not
+                    # centlat_e and centlon_e must be index of maximum, we will loose some inner contour if it's not
                     (
                         max_average_speed,
                         speed_contour,
@@ -782,35 +813,32 @@ class GridDataset(object):
                         pixel_min=pixel_limit[0],
                     )
 
-                    # FIXME : Instantiate new EddyObservation object (high cost need to be review)
+                    # FIXME : Instantiate new EddyObservation object (high cost need to be reviewed)
                     obs = EddiesObservations(
                         size=1,
                         track_extra_variables=track_extra_variables,
                         track_array_variables=sampling,
                         array_variables=array_variables,
                     )
-
-                    obs.obs["height_max_speed_contour"] = self.contours.cvalues[
-                        i_max_speed
-                    ]
-                    obs.obs["height_external_contour"] = cvalues
-                    obs.obs["height_inner_contour"] = self.contours.cvalues[i_inner]
+                    obs.height_max_speed_contour[:] = self.contours.cvalues[i_max_speed]
+                    obs.height_external_contour[:] = cvalues
+                    obs.height_inner_contour[:] = self.contours.cvalues[i_inner]
                     array_size = speed_array.shape[0]
-                    obs.obs["nb_contour_selected"] = array_size
+                    obs.nb_contour_selected[:] = array_size
                     if speed_array.shape[0] == 1:
-                        obs.obs["uavg_profile"][:] = speed_array[0]
+                        obs.uavg_profile[:] = speed_array[0]
                     else:
-                        obs.obs["uavg_profile"] = raw_resample(speed_array, sampling)
-                    obs.obs["amplitude"] = amp.amplitude
-                    obs.obs["speed_average"] = max_average_speed
-                    obs.obs["num_point_e"] = contour.lon.shape[0]
+                        obs.uavg_profile[:] = raw_resample(speed_array, sampling)
+                    obs.amplitude[:] = amp.amplitude
+                    obs.speed_average[:] = max_average_speed
+                    obs.num_point_e[:] = contour.lon.shape[0]
                     xy_e = uniform_resample(contour.lon, contour.lat, **out_sampling)
-                    obs.obs["contour_lon_e"], obs.obs["contour_lat_e"] = xy_e
-                    obs.obs["num_point_s"] = speed_contour.lon.shape[0]
+                    obs.contour_lon_e[:], obs.contour_lat_e[:] = xy_e
+                    obs.num_point_s[:] = speed_contour.lon.shape[0]
                     xy_s = uniform_resample(
                         speed_contour.lon, speed_contour.lat, **out_sampling
                     )
-                    obs.obs["contour_lon_s"], obs.obs["contour_lat_s"] = xy_s
+                    obs.contour_lon_s[:], obs.contour_lat_s[:] = xy_s
 
                     # FIXME : we use a contour without resampling
                     # First, get position based on innermost contour
@@ -821,19 +849,25 @@ class GridDataset(object):
                     centlon_s, centlat_s, eddy_radius_s, aerr_s = _fit_circle_path(
                         create_vertice(*xy_s)
                     )
-                    # Computed again to use resample contour
+                    # Compute again to use resampled contour
                     _, _, eddy_radius_e, aerr_e = _fit_circle_path(
                         create_vertice(*xy_e)
                     )
 
-                    obs.obs["radius_s"] = eddy_radius_s
-                    obs.obs["radius_e"] = eddy_radius_e
-                    obs.obs["shape_error_e"] = aerr_e
-                    obs.obs["shape_error_s"] = aerr_s
-                    obs.obs["lon"] = centlon_s
-                    obs.obs["lat"] = centlat_s
-                    obs.obs["lon_max"] = centlon_i
-                    obs.obs["lat_max"] = centlat_i
+                    obs.radius_s[:] = eddy_radius_s
+                    obs.radius_e[:] = eddy_radius_e
+                    obs.shape_error_e[:] = aerr_e
+                    obs.shape_error_s[:] = aerr_s
+                    obs.speed_area[:] = poly_area(
+                        *coordinates_to_local(*xy_s, lon0=centlon_s, lat0=centlat_s)
+                    )
+                    obs.effective_area[:] = poly_area(
+                        *coordinates_to_local(*xy_e, lon0=centlon_s, lat0=centlat_s)
+                    )
+                    obs.lon[:] = centlon_s
+                    obs.lat[:] = centlat_s
+                    obs.lon_max[:] = centlon_i
+                    obs.lat_max[:] = centlat_i
                     if aerr > 99.9 or aerr_s > 99.9:
                         logger.warning(
                             "Strange shape at this step! shape_error : %f, %f",
@@ -853,18 +887,14 @@ class GridDataset(object):
             else:
                 eddies = EddiesObservations.concatenate(eddies)
             eddies.sign_type = 1 if anticyclonic_search else -1
-            eddies.obs["time"] = (date - datetime(1950, 1, 1)).total_seconds() / 86400.0
+            eddies.time[:] = (date - datetime(1950, 1, 1)).total_seconds() / 86400.0
 
             # normalization longitude between 0 - 360, because storage have an offset on 180
-            eddies.obs["lon_max"] %= 360
-            eddies.obs["lon"] %= 360
-            ref = eddies.obs["lon"] - 180
-            eddies.obs["contour_lon_e"] = (
-                (eddies.obs["contour_lon_e"].T - ref) % 360 + ref
-            ).T
-            eddies.obs["contour_lon_s"] = (
-                (eddies.obs["contour_lon_s"].T - ref) % 360 + ref
-            ).T
+            eddies.lon_max[:] %= 360
+            eddies.lon[:] %= 360
+            ref = eddies.lon - 180
+            eddies.contour_lon_e[:] = ((eddies.contour_lon_e.T - ref) % 360 + ref).T
+            eddies.contour_lon_s[:] = ((eddies.contour_lon_s.T - ref) % 360 + ref).T
             a_and_c.append(eddies)
 
         if in_h_unit is not None:
@@ -923,7 +953,7 @@ class GridDataset(object):
             if not poly_contain_poly(original_contour.vertices, level_contour.vertices):
                 break
             # 3. Respect size range (for max speed)
-            # nb_pixel properties need call of pixels_in before with a grid of pixel
+            # nb_pixel properties need to call pixels_in before with a grid of pixel
             level_contour.pixels_in(self)
             # Interpolate uspd to seglon, seglat, then get mean
             level_average_speed = self.speed_coef_mean(level_contour)
@@ -953,8 +983,7 @@ class GridDataset(object):
 
     @staticmethod
     def _gaussian_filter(data, sigma, mode="reflect"):
-        """Standard gaussian filter
-        """
+        """Standard gaussian filter"""
         local_data = data.copy()
         local_data[data.mask] = 0
 
@@ -966,7 +995,7 @@ class GridDataset(object):
 
     @staticmethod
     def get_amplitude(
-        contour, contour_height, data, anticyclonic_search=True, level=None, step=None
+        contour, contour_height, data, anticyclonic_search=True, level=None, **kwargs
     ):
         # Instantiate Amplitude object
         amp = Amplitude(
@@ -976,21 +1005,17 @@ class GridDataset(object):
             contour_height=contour_height,
             # All grid
             data=data,
-            # Step by level
-            interval=step,
+            **kwargs,
         )
-
         if anticyclonic_search:
             reset_centroid = amp.all_pixels_above_h0(level)
         else:
             reset_centroid = amp.all_pixels_below_h0(level)
-
         return reset_centroid, amp
 
 
 class UnRegularGridDataset(GridDataset):
-    """Class which manage unregular grid
-    """
+    """Class managing unregular grid"""
 
     __slots__ = (
         "index_interp",
@@ -998,8 +1023,7 @@ class UnRegularGridDataset(GridDataset):
     )
 
     def load(self):
-        """Load variable (data)
-        """
+        """Load variable (data)"""
         x_name, y_name = self.coordinates
         with Dataset(self.filename) as h:
             self.x_dim = h.variables[x_name].dimensions
@@ -1017,8 +1041,7 @@ class UnRegularGridDataset(GridDataset):
 
     @property
     def bounds(self):
-        """Give bound
-        """
+        """Give bound"""
         return self.x_c.min(), self.x_c.max(), self.y_c.min(), self.y_c.max()
 
     def bbox_indice(self, vertices):
@@ -1114,8 +1137,7 @@ class UnRegularGridDataset(GridDataset):
 
 
 class RegularGridDataset(GridDataset):
-    """Class only for regular grid
-    """
+    """Class only for regular grid"""
 
     __slots__ = (
         "_speed_ev",
@@ -1161,8 +1183,7 @@ class RegularGridDataset(GridDataset):
         return obj
 
     def init_pos_interpolator(self):
-        """Create function to have a quick index interpolator
-        """
+        """Create function to have a quick index interpolator"""
         self.xinterp = arange(self.x_bounds.shape[0])
         self.yinterp = arange(self.y_bounds.shape[0])
 
@@ -1179,34 +1200,44 @@ class RegularGridDataset(GridDataset):
         )
 
     def get_pixels_in(self, contour):
-        (x_start, x_stop), (y_start, y_stop) = contour.bbox_slice
-        return _get_pixel_in_regular(
-            contour.vertices, self.x_c, self.y_c, x_start, x_stop, y_start, y_stop
-        )
+        """
+        Get indices of pixels in contour.
+
+        :param vertice,Path contour: Contour which enclosed some pixels
+        :return: Indices of grid in contour
+        :rtype: array[int],array[int]
+        """
+        if isinstance(contour, BasePath):
+            (x_start, x_stop), (y_start, y_stop) = contour.bbox_slice
+            return get_pixel_in_regular(
+                contour.vertices, self.x_c, self.y_c, x_start, x_stop, y_start, y_stop
+            )
+        else:
+            (x_start, x_stop), (y_start, y_stop) = self.bbox_indice(contour)
+            return get_pixel_in_regular(
+                contour, self.x_c, self.y_c, x_start, x_stop, y_start, y_stop
+            )
 
     def normalize_x_indice(self, indices):
         return indices % self.x_size
 
     def nearest_grd_indice(self, x, y):
-        return _nearest_grd_indice(
+        return nearest_grd_indice(
             x, y, self.x_bounds, self.y_bounds, self.xstep, self.ystep
         )
 
     @property
     def xstep(self):
-        """Only for regular grid with no step variation
-        """
+        """Only for regular grid with no step variation"""
         return self._x_step
 
     @property
     def ystep(self):
-        """Only for regular grid with no step variation
-        """
+        """Only for regular grid with no step variation"""
         return self._y_step
 
     def compute_pixel_path(self, x0, y0, x1, y1):
-        """Give a series of index which describe the path between to position
-        """
+        """Give a series of indexes which describe the path between to position"""
         return compute_pixel_path(
             x0,
             y0,
@@ -1220,13 +1251,11 @@ class RegularGridDataset(GridDataset):
         )
 
     def clean_land(self):
-        """Function to remove all land pixel
-        """
+        """Function to remove all land pixel"""
         pass
 
     def is_circular(self):
-        """Check if grid is circular
-        """
+        """Check if the grid is circular"""
         if self._is_circular is None:
             self._is_circular = (
                 abs((self.x_bounds[0] % 360) - (self.x_bounds[-1] % 360)) < 0.0001
@@ -1311,8 +1340,7 @@ class RegularGridDataset(GridDataset):
         return self.finalize_kernel(kernel, order, half_x_pt, half_y_pt)
 
     def _low_filter(self, grid_name, w_cut, **kwargs):
-        """low filtering
-        """
+        """low filtering"""
         return self.convolve_filter_with_dynamic_kernel(
             grid_name, self.kernel_bessel, wave_length=w_cut, **kwargs
         )
@@ -1320,6 +1348,15 @@ class RegularGridDataset(GridDataset):
     def convolve_filter_with_dynamic_kernel(
         self, grid, kernel_func, lat_max=85, extend=False, **kwargs_func
     ):
+        """
+        :param str grid: grid name
+        :param func kernel_func: function of kernel to use
+        :param float lat_max: absolute latitude above no filtering apply
+        :param bool extend: if False, only non masked value will return a filtered value
+        :param dict kwargs_func: look at kernel_func
+        :return: filtered value
+        :rtype: array
+        """
         if (abs(self.y_c) > lat_max).any():
             logger.warning("No filtering above %f degrees of latitude", lat_max)
         if isinstance(grid, str):
@@ -1381,7 +1418,7 @@ class RegularGridDataset(GridDataset):
             demi_x, demi_y = k_shape[0] // 2, k_shape[1] // 2
             values_sum = filter2D(tmp_matrix.data, -1, kernel)[demi_x:-demi_x, demi_y]
             kernel_sum = filter2D(m.astype(float), -1, kernel)[demi_x:-demi_x, demi_y]
-            with errstate(invalid="ignore"):
+            with errstate(invalid="ignore", divide="ignore"):
                 if extend:
                     data_out[:, i] = ma.array(
                         values_sum / kernel_sum,
@@ -1412,7 +1449,7 @@ class RegularGridDataset(GridDataset):
             lat_max=lat_max,
             wave_length=wave_length,
             order=order,
-            **kwargs
+            **kwargs,
         )
         self.vars[grid_name] -= data_out
 
@@ -1424,7 +1461,7 @@ class RegularGridDataset(GridDataset):
             lat_max=lat_max,
             wave_length=wave_length,
             order=order,
-            **kwargs
+            **kwargs,
         )
         self.vars[grid_name] = data_out
 
@@ -1439,6 +1476,15 @@ class RegularGridDataset(GridDataset):
         self.vars[grid_name] -= data_out
 
     def bessel_high_filter(self, grid_name, wave_length, order=1, lat_max=85, **kwargs):
+        """
+        :param str grid_name: grid to filter, data will replace original one
+        :param float wave_length: in km
+        :param int order: order to use, if > 1 negative values of the cardinal sinus are present in kernel
+        :param float lat_max: absolute latitude above no filtering apply
+        :param dict kwargs: look at :py:meth:`RegularGridDataset.convolve_filter_with_dynamic_kernel`
+
+        .. minigallery:: py_eddy_tracker.RegularGridDataset.bessel_high_filter
+        """
         logger.debug(
             "Run filtering with wave of %(wave_length)s km and order of %(order)s ...",
             dict(wave_length=wave_length, order=order),
@@ -1449,7 +1495,7 @@ class RegularGridDataset(GridDataset):
             lat_max=lat_max,
             wave_length=wave_length,
             order=order,
-            **kwargs
+            **kwargs,
         )
         logger.debug("Filtering done")
         self.vars[grid_name] -= data_out
@@ -1461,7 +1507,7 @@ class RegularGridDataset(GridDataset):
             lat_max=lat_max,
             wave_length=wave_length,
             order=order,
-            **kwargs
+            **kwargs,
         )
         self.vars[grid_name] = data_out
 
@@ -1565,6 +1611,76 @@ class RegularGridDataset(GridDataset):
     def compute_stencil(
         self, data, stencil_halfwidth=4, mode="reflect", vertical=False
     ):
+        r"""
+        Apply stencil ponderation on field.
+
+        :param array data: array where apply stencil
+        :param int stencil_halfwidth: from 1 t0 4, maximal stencil used
+        :param str mode: convolution mode
+        :param bool vertical: if True, method apply a vertical convolution
+        :return: gradient array from stencil application
+        :rtype: array
+
+        Short story, how to get stencil coefficient for stencil (3 points, 5 points and 7 points)
+
+        Taylor's theorem:
+
+        .. math::
+            f(x \pm h) = f(x) \pm f'(x)h
+                + \frac{f''(x)h^2}{2!} \pm \frac{f^{(3)}(x)h^3}{3!}
+                + \frac{f^{(4)}(x)h^4}{4!} \pm \frac{f^{(5)}(x)h^5}{5!}
+                + O(h^6)
+
+        If we stop at `O(h^2)`, we get classic differenciation (stencil 3 points):
+
+        .. math:: f(x+h) - f(x-h) = f(x) - f(x) + 2 f'(x)h + O(h^2)
+
+        .. math:: f'(x) = \frac{f(x+h) - f(x-h)}{2h} + O(h^2)
+
+        If we stop at `O(h^4)`, we will get stencil 5 points:
+
+        .. math::
+            f(x+h) - f(x-h) = 2 f'(x)h + 2 \frac{f^{(3)}(x)h^3}{3!} + O(h^4)
+            :label: E1
+
+        .. math::
+            f(x+2h) - f(x-2h) = 4 f'(x)h + 16 \frac{f^{(3)}(x)h^3}{3!} + O(h^4)
+            :label: E2
+
+        If we multiply equation :eq:`E1` by 8 and substract equation :eq:`E2`, we get:
+
+        .. math:: 8(f(x+h) - f(x-h)) - (f(x+2h) - f(x-2h)) = 16 f'(x)h - 4 f'(x)h + O(h^4)
+
+        .. math:: f'(x) = \frac{f(x-2h) - 8f(x-h) + 8f(x+h) - f(x+2h)}{12h} + O(h^4)
+
+        If we stop at `O(h^6)`, we will get stencil 7 points:
+
+        .. math::
+            f(x+h) - f(x-h) = 2 f'(x)h + 2 \frac{f^{(3)}(x)h^3}{3!} + 2 \frac{f^{(5)}(x)h^5}{5!} + O(h^6)
+            :label: E3
+
+        .. math::
+            f(x+2h) - f(x-2h) = 4 f'(x)h + 16 \frac{f^{(3)}(x)h^3}{3!} + 64 \frac{f^{(5)}(x)h^5}{5!} + O(h^6)
+            :label: E4
+
+        .. math::
+            f(x+3h) - f(x-3h) = 6 f'(x)h + 54 \frac{f^{(3)}(x)h^3}{3!} + 486 \frac{f^{(5)}(x)h^5}{5!} + O(h^6)
+            :label: E5
+
+        If we multiply equation :eq:`E3` by 45 and substract equation :eq:`E4` multiply by 9
+        and add equation :eq:`E5`, we get:
+
+        .. math::
+            45(f(x+h) - f(x-h)) - 9(f(x+2h) - f(x-2h)) + (f(x+3h) - f(x-3h)) =
+            90 f'(x)h - 36 f'(x)h + 6 f'(x)h + O(h^6)
+
+        .. math::
+            f'(x) = \frac{-f(x-3h) + 9f(x-2h) - 45f(x-h) + 45f(x+h) - 9f(x+2h) +f(x+3h)}{60h} + O(h^6)
+
+        ...
+
+
+        """
         stencil_halfwidth = max(min(int(stencil_halfwidth), 4), 1)
         logger.debug("Stencil half width apply : %d", stencil_halfwidth)
         # output
@@ -1691,7 +1807,14 @@ class RegularGridDataset(GridDataset):
 
     def add_uv(self, grid_height, uname="u", vname="v", stencil_halfwidth=4):
         """Compute a u and v grid
-               """
+
+        :param str grid_height: grid name where the funtion will apply stencil method
+        :param str uname: future name of u
+        :param str vname: future name of v
+        :param int stencil_halfwidth: largest stencil could be apply (max: 4)
+
+        .. minigallery:: py_eddy_tracker.RegularGridDataset.add_uv
+        """
         logger.info("Add u/v variable with stencil method")
         data = self.grid(grid_height)
         h_dict = self.variables_description[grid_height]
@@ -1734,7 +1857,7 @@ class RegularGridDataset(GridDataset):
         )
 
     def speed_coef_mean(self, contour):
-        """some nan can be compute over contour if we are near border,
+        """Some nan can be computed over contour if we are near border,
         something to explore
         """
         return mean_on_regular_contour(
@@ -1747,24 +1870,83 @@ class RegularGridDataset(GridDataset):
         )
 
     def init_speed_coef(self, uname="u", vname="v"):
-        """Draft
-        """
+        """Draft"""
         self._speed_ev = (self.grid(uname) ** 2 + self.grid(vname) ** 2) ** 0.5
 
-    def display(self, ax, name, factor=1, **kwargs):
+    def display(self, ax, name, factor=1, ref=None, **kwargs):
+        """
+        :param matplotlib.axes.Axes ax: matplotlib axes use to draw
+        :param str,array name: variable to display, could be an array
+        :param float factor: multiply grid by
+        :param float,None ref: if define use like west bound
+        :param dict kwargs: look at :py:meth:`matplotlib.axes.Axes.pcolormesh`
+
+        .. minigallery:: py_eddy_tracker.RegularGridDataset.display
+        """
         if "cmap" not in kwargs:
             kwargs["cmap"] = "coolwarm"
-        return ax.pcolormesh(
-            self.x_bounds, self.y_bounds, self.grid(name).T * factor, **kwargs
-        )
+        data = self.grid(name) if isinstance(name, str) else name
+        if ref is None:
+            x = self.x_bounds
+        else:
+            x = (self.x_c - ref) % 360 + ref
+            i = x.argsort()
+            x = self.c_to_bounds(x[i])
+            data = data[i]
+        return ax.pcolormesh(x, self.y_bounds, data.T * factor, **kwargs)
 
-    def interp(self, grid_name, lons, lats):
+    def contour(self, ax, name, factor=1, ref=None, **kwargs):
+        """
+        :param matplotlib.axes.Axes ax: matplotlib axes use to draw
+        :param str,array name: variable to display, could be an array
+        :param float factor: multiply grid by
+        :param float,None ref: if define use like west bound
+        :param dict kwargs: look at :py:meth:`matplotlib.axes.Axes.contour`
+
+        .. minigallery:: py_eddy_tracker.RegularGridDataset.contour
+        """
+        data = self.grid(name) if isinstance(name, str) else name
+        if ref is None:
+            x = self.x_c
+        else:
+            x = (self.x_c - ref) % 360 + ref
+            i = x.argsort()
+            x = x[i]
+            data = data[i]
+        return ax.contour(x, self.y_c, data.T * factor, **kwargs)
+
+    def regrid(self, other, grid_name, new_name=None):
+        """
+        Interpolate another grid at the current grid position
+
+        :param RegularGridDataset other:
+        :param str grid_name: variable name to interpolate
+        :param str new_name: name used to store, if None method will use current ont
+
+        .. minigallery:: py_eddy_tracker.RegularGridDataset.regrid
+        """
+        if new_name is None:
+            new_name = grid_name
+        x, y = meshgrid(self.x_c, self.y_c)
+        # interp and reshape
+        v_interp = (
+            other.interp(grid_name, x.reshape(-1), y.reshape(-1)).reshape(x.shape).T
+        )
+        v_interp = ma.array(v_interp, mask=isnan(v_interp))
+        # and add it to self
+        self.add_grid(new_name, v_interp)
+        self.variables_description[new_name] = other.variables_description[grid_name]
+        # self.variables_description[new_name]['infos'] = False
+        # self.variables_description[new_name]['kwargs']['dimensions'] = ...
+
+    def interp(self, grid_name, lons, lats, method="bilinear"):
         """
         Compute z over lons, lats
 
-        :param str grid_name: Grid which will be interp
+        :param str grid_name: Grid to be interpolated
         :param lons: new x
         :param lats: new y
+        :param str method: Could be 'bilinear' or 'nearest'
 
         :return: new z
         """
@@ -1773,13 +1955,14 @@ class RegularGridDataset(GridDataset):
             m = g.mask
         else:
             m = ones(g.shape) if g.mask else zeros(g.shape)
-        return interp2d_geo(self.x_c, self.y_c, g, m, lons, lats)
+        return interp2d_geo(
+            self.x_c, self.y_c, g, m, lons, lats, nearest=method == "nearest"
+        )
 
 
 @njit(cache=True, fastmath=True)
 def compute_pixel_path(x0, y0, x1, y1, x_ori, y_ori, x_step, y_step, nb_x):
-    """Give a series of index which describe the path between to position
-    """
+    """Give a serie of indexes describing the path between two position"""
     # index
     nx = x0.shape[0]
     i_x0 = empty(nx, dtype=numba_types.int_)
@@ -1846,29 +2029,6 @@ def compute_pixel_path(x0, y0, x1, y1, x_ori, y_ori, x_step, y_step, nb_x):
     return i_g, j_g, d_max
 
 
-@njit(cache=True)
-def bbox_indice_regular(vertices, x0, y0, xstep, ystep, N, circular, x_size):
-    lon, lat = vertices[:, 0], vertices[:, 1]
-    lon_min, lon_max = lon.min(), lon.max()
-    lat_min, lat_max = lat.min(), lat.max()
-    i_x0, i_y0 = _nearest_grd_indice(lon_min, lat_min, x0, y0, xstep, ystep)
-    i_x1, i_y1 = _nearest_grd_indice(lon_max, lat_max, x0, y0, xstep, ystep)
-    if circular:
-        slice_x = (i_x0 - N) % x_size, (i_x1 + N + 1) % x_size
-    else:
-        slice_x = max(i_x0 - N, 0), i_x1 + N + 1
-    slice_y = i_y0 - N, i_y1 + N + 1
-    return slice_x, slice_y
-
-
-@njit(cache=True, fastmath=True)
-def _nearest_grd_indice(x, y, x0, y0, xstep, ystep):
-    return (
-        numba_types.int32(round(((x - x0[0]) % 360.0) / xstep)),
-        numba_types.int32(round((y - y0[0]) / ystep)),
-    )
-
-
 @njit(cache=True)
 def has_masked_value(grid, i_x, i_y):
     for i, j in zip(i_x, i_y):
