@@ -1,30 +1,28 @@
 # -*- coding: utf-8 -*-
 """
-===========================================================================
-This file is part of py-eddy-tracker.
-
-    py-eddy-tracker is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    py-eddy-tracker is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with py-eddy-tracker.  If not, see <http://www.gnu.org/licenses/>.
-
-Copyright (c) 2014-2020 by Evan Mason
-Email: evanmason@gmail.com
-===========================================================================
+Class to compute Amplitude and average speed profile
 """
 
 import logging
-from numpy import empty, array, concatenate, ma, zeros, unique, round, ones, int_
+
+from matplotlib.cm import get_cmap
+from matplotlib.colors import Normalize
 from matplotlib.figure import Figure
-from numba import njit, types as numba_types
+from numba import njit
+from numba import types as numba_types
+from numpy import (
+    array,
+    concatenate,
+    digitize,
+    empty,
+    int_,
+    ma,
+    ones,
+    round,
+    unique,
+    zeros,
+)
+
 from .poly import winding_number_poly
 
 logger = logging.getLogger("pet")
@@ -45,15 +43,38 @@ class Amplitude(object):
         "sla",
         "contour",
         "interval_min",
+        "interval_min_secondary",
         "amplitude",
         "mle",
     )
 
-    def __init__(self, contour, contour_height, data, interval):
+    def __init__(
+        self,
+        contour,
+        contour_height,
+        data,
+        interval,
+        mle=1,
+        nb_step_min=2,
+        nb_step_to_be_mle=2,
+    ):
+        """
+        Create amplitude object
+
+        :param Contours contour:
+        :param float contour_height:
+        :param array data:
+        :param float interval:
+        :param int mle: maximum number of local maxima in contour
+        :param int nb_step_min: number of interval to consider like an eddy
+        :param int nb_step_to_be_mle: number of interval to be consider like another maxima
+        """
+
         # Height of the contour
         self.h_0 = contour_height
         # Step minimal to consider amplitude
-        self.interval_min = interval * 2
+        self.interval_min = interval * nb_step_min
+        self.interval_min_secondary = interval * nb_step_to_be_mle
         # Indices of all pixels in contour
         self.contour = contour
         # Link on original grid (local view) or copy if it's on bound
@@ -85,11 +106,10 @@ class Amplitude(object):
         # Amplitude which will be provide
         self.amplitude = 0
         # Maximum local extrema accepted
-        self.mle = 1
+        self.mle = mle
 
     def within_amplitude_limits(self):
-        """Need update
-        """
+        """Need update"""
         return self.interval_min <= self.amplitude
 
     def all_pixels_below_h0(self, level):
@@ -123,7 +143,8 @@ class Amplitude(object):
             else:
                 # Verify if several extrema are seriously below contour
                 nb_real_extrema = (
-                    (level - self.grid_extract.data[lmi_i, lmi_j]) >= self.interval_min
+                    (level - self.grid_extract.data[lmi_i, lmi_j])
+                    >= self.interval_min_secondary
                 ).sum()
                 if nb_real_extrema > self.mle:
                     return False
@@ -165,7 +186,8 @@ class Amplitude(object):
             else:
                 # Verify if several extrema are seriously above contour
                 nb_real_extrema = (
-                    (self.grid_extract.data[lmi_i, lmi_j] - level) >= self.interval_min
+                    (self.grid_extract.data[lmi_i, lmi_j] - level)
+                    >= self.interval_min_secondary
                 ).sum()
                 if nb_real_extrema > self.mle:
                     return False
@@ -409,8 +431,8 @@ class Contours(object):
         closed_contours = 0
         # Count level and contour
         for i, collection in enumerate(self.contours.collections):
-            collection.get_nearest_path_bbox_contain_pt = lambda x, y, i=i: self.get_index_nearest_path_bbox_contain_pt(
-                i, x, y
+            collection.get_nearest_path_bbox_contain_pt = (
+                lambda x, y, i=i: self.get_index_nearest_path_bbox_contain_pt(i, x, y)
             )
             nb_level += 1
 
@@ -420,17 +442,14 @@ class Contours(object):
                 # Contour with less vertices than 4 are popped
                 if contour.vertices.shape[0] < 4:
                     continue
-                if keep_unclose:
-                    keep_path.append(contour)
-                    continue
                 # Remove unclosed path
                 d_closed = (
                     (contour.vertices[0, 0] - contour.vertices[-1, 0]) ** 2
                     + (contour.vertices[0, 1] - contour.vertices[-1, 1]) ** 2
                 ) ** 0.5
-                if d_closed > self.DELTA_SUP:
+                if d_closed > self.DELTA_SUP and not keep_unclose:
                     continue
-                elif d_closed != 0:
+                elif d_closed != 0 and d_closed <= self.DELTA_SUP:
                     # Repair almost closed contour
                     if d_closed > self.DELTA_PREC:
                         almost_closed_contours += 1
@@ -451,6 +470,7 @@ class Contours(object):
             collection._paths = keep_path
             for contour in collection.get_paths():
                 contour.used = False
+                contour.reject = 0
                 nb_contour += 1
                 nb_pt += contour.vertices.shape[0]
         logger.info(
@@ -555,12 +575,51 @@ class Contours(object):
         only_used=False,
         only_unused=False,
         only_contain_eddies=False,
+        display_criterion=False,
+        field=None,
+        bins=None,
+        cmap="Spectral_r",
         **kwargs
     ):
+        """
+        Display contour
+
+        :param matplotlib.axes.Axes ax:
+        :param int step: display only contour every step
+        :param bool only_used: display only contour used in an eddy
+        :param bool only_unused: display only contour unused in an eddy
+        :param bool only_contain_eddies: display only contour which enclosed an eddiy
+        :param bool display_criterion:
+            display only unused contour with criterion color
+
+            0. - Accepted (green)
+            1. - Reject for shape error (red)
+            2. - Masked value in contour (blue)
+            3. - Under or over pixel limit bound (black)
+            4. - Amplitude criterion (yellow)
+        :param str field:
+            Must be 'shape_error', 'x', 'y' or 'radius'.
+            If define display_criterion is not use.
+            bins argument must be define
+        :param array bins: bins use to colorize contour
+        :param str cmap: Name of cmap to use for field display
+        :param dict kwargs: look at :py:meth:`matplotlib.collections.LineCollection`
+
+        .. minigallery:: py_eddy_tracker.Contours.display
+        """
         from matplotlib.collections import LineCollection
 
+        overide_color = display_criterion or field is not None
+        if display_criterion:
+            paths = {0: list(), 1: list(), 2: list(), 3: list(), 4: list()}
+        elif field is not None:
+            paths = dict()
+            for i in range(len(bins)):
+                paths[i] = list()
+            paths[i + 1] = list()
         for j, collection in enumerate(self.contours.collections[::step]):
-            paths = list()
+            if not overide_color:
+                paths = list()
             for i in collection.get_paths():
                 if only_used and not i.used:
                     continue
@@ -568,18 +627,58 @@ class Contours(object):
                     continue
                 elif only_contain_eddies and not i.contain_eddies:
                     continue
-                paths.append(i.vertices)
+                if display_criterion:
+                    paths[i.reject].append(i.vertices)
+                elif field is not None:
+                    x, y, radius, shape_error = i.fit_circle()
+                    if field == "shape_error":
+                        i_ = digitize(shape_error, bins)
+                    elif field == "radius":
+                        i_ = digitize(radius, bins)
+                    elif field == "x":
+                        i_ = digitize(x, bins)
+                    elif field == "y":
+                        i_ = digitize(y, bins)
+                    paths[i_].append(i.vertices)
+                else:
+                    paths.append(i.vertices)
             local_kwargs = kwargs.copy()
             if "color" not in kwargs:
                 local_kwargs["color"] = collection.get_color()
                 local_kwargs.pop("label", None)
             elif j != 0:
                 local_kwargs.pop("label", None)
-            ax.add_collection(LineCollection(paths, **local_kwargs))
-
-        if hasattr(self.contours, "_mins"):
-            ax.update_datalim([self.contours._mins, self.contours._maxs])
-        ax.autoscale_view()
+            if not overide_color:
+                ax.add_collection(LineCollection(paths, **local_kwargs))
+        if display_criterion:
+            colors = {0: "g", 1: "r", 2: "b", 3: "k", 4: "y"}
+            for k, v in paths.items():
+                local_kwargs = kwargs.copy()
+                local_kwargs.pop("label", None)
+                local_kwargs["colors"] = colors[k]
+                ax.add_collection(LineCollection(v, **local_kwargs))
+        elif field is not None:
+            nb_bins = len(bins) - 1
+            cmap = get_cmap(cmap, lut=nb_bins)
+            for k, v in paths.items():
+                local_kwargs = kwargs.copy()
+                local_kwargs.pop("label", None)
+                if k == 0:
+                    local_kwargs["colors"] = cmap(0.0)
+                elif k > nb_bins:
+                    local_kwargs["colors"] = cmap(1.0)
+                else:
+                    local_kwargs["colors"] = cmap((k - 1.0) / nb_bins)
+                mappable = LineCollection(v, **local_kwargs)
+                ax.add_collection(mappable)
+            mappable.cmap = cmap
+            mappable.norm = Normalize(vmin=bins[0], vmax=bins[-1])
+            # TODO : need to create an object with all collections
+            return mappable
+        else:
+            if hasattr(self.contours, "_mins"):
+                ax.update_datalim([self.contours._mins, self.contours._maxs])
+            ax.autoscale_view()
 
     def label_contour_unused_which_contain_eddies(self, eddies):
         """Select contour which contain several eddies"""
@@ -634,8 +733,7 @@ def index_from_nearest_path_with_pt_in_bbox_(
     xpt,
     ypt,
 ):
-    """Get index from nearest path in edge bbox contain pt
-    """
+    """Get index from nearest path in edge bbox contain pt"""
     # Nb contour in level
     if nb_c_per_l[level_index] == 0:
         return -1
