@@ -1,50 +1,62 @@
 # -*- coding: utf-8 -*-
 """
-===========================================================================
-This file is part of py-eddy-tracker.
-
-    py-eddy-tracker is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    py-eddy-tracker is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with py-eddy-tracker.  If not, see <http://www.gnu.org/licenses/>.
-
-Copyright (c) 2014-2020 by Evan Mason
-Email: evanmason@gmail.com
-===========================================================================
+Tool method which use mostly numba
 """
 
+from numba import njit, prange
+from numba import types as numba_types
 from numpy import (
-    sin,
-    pi,
-    cos,
-    arctan2,
-    arcsin,
-    empty,
-    nan,
     absolute,
+    arcsin,
+    arctan2,
+    bool_,
+    cos,
+    empty,
     floor,
-    ones,
-    linspace,
+    histogram,
     interp,
+    isnan,
+    linspace,
+    nan,
+    ones,
+    pi,
+    radians,
+    sin,
     where,
     zeros,
-    isnan,
-    bool_,
 )
-from numba import njit, prange, types as numba_types
-from numpy.linalg import lstsq
+
+
+@njit(cache=True)
+def count_consecutive(mask):
+    """
+    Count consecutive event every False flag count restart
+
+    :param array[bool] mask: event to count
+    :return: count when consecutive event
+    :rtype: array
+    """
+    count = 0
+    output = zeros(mask.shape, dtype=numba_types.int_)
+    for i in range(mask.shape[0]):
+        if not mask[i]:
+            count = 0
+            continue
+        count += 1
+        output[i] = count
+    return output
 
 
 @njit(cache=True)
 def reverse_index(index, nb):
+    """
+    Compute a list of index, which are not in index.
+
+    :param array index: index of group which will be set to False
+    :param array nb: Count for each group
+    :return: mask of value selected
+    :rtype: array
+    """
     m = ones(nb, dtype=numba_types.bool_)
     for i in index:
         m[i] = False
@@ -53,7 +65,11 @@ def reverse_index(index, nb):
 
 @njit(cache=True)
 def build_index(groups):
-    """We expected that variable is monotonous, and return index for each step change
+    """We expected that variable is monotonous, and return index for each step change.
+
+    :param array groups: array which contain group to be separated
+    :return: (first_index of each group, last_index of each group, value to shift group)
+    :rtype: (array, array, int)
     """
     i0, i1 = groups.min(), groups.max()
     amplitude = i1 - i0 + 1
@@ -71,17 +87,24 @@ def build_index(groups):
     return first_index, last_index, i0
 
 
+@njit(cache=True)
+def hist_numba(x, bins):
+    """Call numba histogram  to speed up."""
+    return histogram(x, bins)
+
+
 @njit(cache=True, fastmath=True, parallel=False)
 def distance_grid(lon0, lat0, lon1, lat1):
     """
-    Args:
-        lon0:
-        lat0:
-        lon1:
-        lat1:
+    Get distance for every couple of point.
 
-    Returns:
-        nan value for far away point, and km for other
+    :param array lon0:
+    :param array lat0:
+    :param array lon1:
+    :param array lat1:
+
+    :return: nan value for far away point, and km for other
+    :rtype: array
     """
     nb_0 = lon0.shape[0]
     nb_1 = lon1.shape[0]
@@ -110,6 +133,16 @@ def distance_grid(lon0, lat0, lon1, lat1):
 
 @njit(cache=True, fastmath=True)
 def distance(lon0, lat0, lon1, lat1):
+    """
+    Compute distance between points from each line.
+
+    :param float lon0:
+    :param float lat0:
+    :param float lon1:
+    :param float lat1:
+    :return: distance (in m)
+    :rtype: array
+    """
     D2R = pi / 180.0
     sin_dlat = sin((lat1 - lat0) * 0.5 * D2R)
     sin_dlon = sin((lon1 - lon0) * 0.5 * D2R)
@@ -120,36 +153,51 @@ def distance(lon0, lat0, lon1, lat1):
 
 
 @njit(cache=True)
-def distance_vincenty(lon0, lat0, lon1, lat1):
-    """ better than haversine but buggy ??"""
-    D2R = pi / 180.0
-    dlon = (lon1 - lon0) * D2R
-    cos_dlon = cos(dlon)
-    cos_lat1 = cos(lat0 * D2R)
-    cos_lat2 = cos(lat1 * D2R)
-    sin_lat1 = sin(lat0 * D2R)
-    sin_lat2 = sin(lat1 * D2R)
-    return 6370997.0 * arctan2(
-        (
-            (cos_lat2 * sin(dlon) ** 2)
-            + (cos_lat1 * sin_lat2 - sin_lat1 * cos_lat2 * cos_dlon) ** 2
-        )
-        ** 0.5,
-        sin_lat1 * sin_lat2 + cos_lat1 * cos_lat2 * cos_dlon,
-    )
+def cumsum_by_track(field, track):
+    """
+    Cumsum by track.
+
+    :param array field: data to sum
+    :pram array(int) track: id of track to separate data
+    :return: cumsum with a reset at each start of track
+    :rtype: array
+    """
+    tr_previous = 0
+    d_cum = 0
+    cumsum_array = empty(track.shape, dtype=field.dtype)
+    for i in range(field.shape[0]):
+        tr = track[i]
+        if tr != tr_previous:
+            d_cum = 0
+        d_cum += field[i]
+        cumsum_array[i] = d_cum
+        tr_previous = tr
+    return cumsum_array
 
 
 @njit(cache=True, fastmath=True)
-def interp2d_geo(x_g, y_g, z_g, m_g, x, y):
-    """For geographic grid, test of cicularity
-    Maybe test if we are out of bounds
+def interp2d_geo(x_g, y_g, z_g, m_g, x, y, nearest=False):
     """
+    For geographic grid, test of cicularity.
+
+    :param array x_g: coordinates of grid
+    :param array y_g: coordinates of grid
+    :param array z_g: Grid value
+    :param array m_g: Boolean grid, True if value is masked
+    :param array x: coordinate where interpolate z
+    :param array y: coordinate where interpolate z
+    :param bool nearest: if true we will take nearest pixel
+    :return: z interpolated
+    :rtype: array
+    """
+    # TODO : Maybe test if we are out of bounds
     x_ref = x_g[0]
     y_ref = y_g[0]
     x_step = x_g[1] - x_ref
     y_step = y_g[1] - y_ref
     nb_x = x_g.shape[0]
-    is_circular = (x_g[-1] + x_step) % 360 == x_g[0] % 360
+    nb_y = y_g.shape[0]
+    is_circular = abs(x_g[-1] % 360 - (x_g[0] - x_step) % 360) < 1e-5
     z = empty(x.shape, dtype=z_g.dtype)
     for i in prange(x.size):
         x_ = (x[i] - x_ref) / x_step
@@ -157,11 +205,16 @@ def interp2d_geo(x_g, y_g, z_g, m_g, x, y):
         i0 = int(floor(x_))
         i1 = i0 + 1
         xd = x_ - i0
+        j0 = int(floor(y_))
+        j1 = j0 + 1
         if is_circular:
             i0 %= nb_x
             i1 %= nb_x
-        j0 = int(floor(y_))
-        j1 = j0 + 1
+        else:
+            if i1 >= nb_x or i0 < 0 or j0 < 0 or j1 >= nb_y:
+                z[i] = nan
+                continue
+
         yd = y_ - j0
         z00 = z_g[i0, j0]
         z01 = z_g[i0, j1]
@@ -170,110 +223,33 @@ def interp2d_geo(x_g, y_g, z_g, m_g, x, y):
         if m_g[i0, j0] or m_g[i0, j1] or m_g[i1, j0] or m_g[i1, j1]:
             z[i] = nan
         else:
-            z[i] = (z00 * (1 - xd) + (z10 * xd)) * (1 - yd) + (
-                z01 * (1 - xd) + z11 * xd
-            ) * yd
-    return z
-
-
-@njit(cache=True, fastmath=True, parallel=False)
-def custom_convolution(data, mask, kernel):
-    """do sortin at high lattitude big part of value are masked"""
-    nb_x = kernel.shape[0]
-    demi_x = int((nb_x - 1) / 2)
-    demi_y = int((kernel.shape[1] - 1) / 2)
-    out = empty(data.shape[0] - nb_x + 1)
-    for i in prange(out.shape[0]):
-        if mask[i + demi_x, demi_y] == 1:
-            w = (mask[i : i + nb_x] * kernel).sum()
-            if w != 0:
-                out[i] = (data[i : i + nb_x] * kernel).sum() / w
+            if nearest:
+                if xd <= 0.5:
+                    if yd <= 0.5:
+                        z[i] = z00
+                    else:
+                        z[i] = z01
+                else:
+                    if yd <= 0.5:
+                        z[i] = z10
+                    else:
+                        z[i] = z11
             else:
-                out[i] = nan
-        else:
-            out[i] = nan
-    return out
-
-
-@njit(cache=True)
-def fit_circle(x_vec, y_vec):
-    nb_elt = x_vec.shape[0]
-    p_inon_x = empty(nb_elt)
-    p_inon_y = empty(nb_elt)
-
-    # last coordinates == first
-    x_mean = x_vec[1:].mean()
-    y_mean = y_vec[1:].mean()
-
-    norme = (x_vec[1:] - x_mean) ** 2 + (y_vec[1:] - y_mean) ** 2
-    norme_max = norme.max()
-    scale = norme_max ** 0.5
-
-    # Form matrix equation and solve it
-    # Maybe put f4
-    datas = ones((nb_elt - 1, 3))
-    datas[:, 0] = 2.0 * (x_vec[1:] - x_mean) / scale
-    datas[:, 1] = 2.0 * (y_vec[1:] - y_mean) / scale
-
-    (center_x, center_y, radius), _, _, _ = lstsq(datas, norme / norme_max)
-
-    # Unscale data and get circle variables
-    radius += center_x ** 2 + center_y ** 2
-    radius **= 0.5
-    center_x *= scale
-    center_y *= scale
-    # radius of fitted circle
-    radius *= scale
-    # center X-position of fitted circle
-    center_x += x_mean
-    # center Y-position of fitted circle
-    center_y += y_mean
-
-    # area of fitted circle
-    c_area = (radius ** 2) * pi
-    # Find distance between circle center and contour points_inside_poly
-    for i_elt in range(nb_elt):
-        # Find distance between circle center and contour points_inside_poly
-        dist_poly = (
-            (x_vec[i_elt] - center_x) ** 2 + (y_vec[i_elt] - center_y) ** 2
-        ) ** 0.5
-        # Indices of polygon points outside circle
-        # p_inon_? : polygon x or y points inside & on the circle
-        if dist_poly > radius:
-            p_inon_y[i_elt] = center_y + radius * (y_vec[i_elt] - center_y) / dist_poly
-            p_inon_x[i_elt] = center_x - (center_x - x_vec[i_elt]) * (
-                center_y - p_inon_y[i_elt]
-            ) / (center_y - y_vec[i_elt])
-        else:
-            p_inon_x[i_elt] = x_vec[i_elt]
-            p_inon_y[i_elt] = y_vec[i_elt]
-
-    # Area of closed contour/polygon enclosed by the circle
-    p_area_incirc = 0
-    p_area = 0
-    for i_elt in range(nb_elt - 1):
-        # Indices of polygon points outside circle
-        # p_inon_? : polygon x or y points inside & on the circle
-        p_area_incirc += (
-            p_inon_x[i_elt] * p_inon_y[1 + i_elt]
-            - p_inon_x[i_elt + 1] * p_inon_y[i_elt]
-        )
-        # Shape test
-        # Area and centroid of closed contour/polygon
-        p_area += x_vec[i_elt] * y_vec[1 + i_elt] - x_vec[1 + i_elt] * y_vec[i_elt]
-    p_area = abs(p_area) * 0.5
-    p_area_incirc = abs(p_area_incirc) * 0.5
-
-    a_err = (c_area - 2 * p_area_incirc + p_area) * 100.0 / c_area
-    return center_x, center_y, radius, a_err
+                z[i] = (z00 * (1 - xd) + (z10 * xd)) * (1 - yd) + (
+                    z01 * (1 - xd) + z11 * xd
+                ) * yd
+    return z
 
 
 @njit(cache=True, fastmath=True)
 def uniform_resample(x_val, y_val, num_fac=2, fixed_size=None):
     """
-    Resample contours to have (nearly) equal spacing
-       x_val, y_val : input contour coordinates
-       num_fac : factor to increase lengths of output coordinates
+    Resample contours to have (nearly) equal spacing.
+
+    :param array_like x_val: input x contour coordinates
+    :param array_like y_val: input y contour coordinates
+    :param int num_fac: factor to increase lengths of output coordinates
+    :param int,None fixed_size: if define, it will used to set sampling
     """
     nb = x_val.shape[0]
     # Get distances
@@ -295,15 +271,21 @@ def uniform_resample(x_val, y_val, num_fac=2, fixed_size=None):
 @njit(cache=True)
 def flatten_line_matrix(l_matrix):
     """
-    Flat matrix and add on between each line
-    Args:
-        l_matrix: matrix of position
+    Flat matrix and add on between each line.
 
-    Returns: array with nan between line
+    :param l_matrix: matrix of position
+    :return: array with nan between line
     """
     nb_line, sampling = l_matrix.shape
     final_size = (nb_line - 1) + nb_line * sampling
+    empty_dataset = False
+    if final_size < 1:
+        empty_dataset = True
+        final_size = 1
     out = empty(final_size, dtype=l_matrix.dtype)
+    if empty_dataset:
+        out[:] = nan
+        return out
     inc = 0
     for i in range(nb_line):
         for j in range(sampling):
@@ -316,12 +298,39 @@ def flatten_line_matrix(l_matrix):
 
 @njit(cache=True)
 def simplify(x, y, precision=0.1):
+    """
+    Will remove all middle/end point which are closer than precision.
+
+    :param array x:
+    :param array y:
+    :param float precision: if two points have distance inferior to precision with remove next point
+    :return: (x,y)
+    :rtype: (array,array)
+    """
     precision2 = precision ** 2
     nb = x.shape[0]
-    x_previous, y_previous = x[0], y[0]
+    # will be True for value keep
     mask = ones(nb, dtype=bool_)
-    for i in range(1, nb):
+    for j in range(0, nb):
+        x_previous, y_previous = x[j], y[j]
+        if isnan(x_previous) or isnan(y_previous):
+            mask[j] = False
+            continue
+        break
+    # Only nan
+    if j == (nb - 1):
+        return zeros(0, dtype=x.dtype), zeros(0, dtype=x.dtype)
+
+    last_nan = False
+    for i in range(j + 1, nb):
         x_, y_ = x[i], y[i]
+        if isnan(x_) or isnan(y_):
+            if last_nan:
+                mask[i] = False
+            else:
+                last_nan = True
+            continue
+        last_nan = False
         d_x = x_ - x_previous
         if d_x > precision:
             x_previous, y_previous = x_, y_
@@ -348,13 +357,13 @@ def simplify(x, y, precision=0.1):
 @njit(cache=True)
 def split_line(x, y, i):
     """
-    Split x and y at each i change
-    Args:
-        x: array
-        y: array
-        i: array of int at each i change, we cut x, y
+    Split x and y at each i change.
 
-    Returns: x and y separate by nan at each i jump
+    :param x: array
+    :param y: array
+    :param i: array of int at each i change, we cut x, y
+
+    :return: x and y separate by nan at each i jump
     """
     nb_jump = len(where(i[1:] - i[:-1] != 0)[0])
     nb_value = x.shape[0]
@@ -375,6 +384,16 @@ def split_line(x, y, i):
 
 @njit(cache=True)
 def wrap_longitude(x, y, ref, cut=False):
+    """
+    Will wrap contiguous longitude with reference as west bound.
+
+    :param array x:
+    :param array y:
+    :param float ref: longitude of reference, all the new value will be between ref and ref + 360
+    :param bool cut: if True line will be cut at the bounds
+    :return: lon,lat
+    :rtype: (array,array)
+    """
     if cut:
         indexs = list()
         nb = x.shape[0]
@@ -421,6 +440,16 @@ def wrap_longitude(x, y, ref, cut=False):
 
 @njit(cache=True, fastmath=True)
 def coordinates_to_local(lon, lat, lon0, lat0):
+    """
+    Take latlong coordinates to transform in local coordinates (in m).
+
+    :param array x: coordinates to transform
+    :param array y: coordinates to transform
+    :param float lon0: longitude of local reference
+    :param float lat0: latitude of local reference
+    :return: x,y
+    :retype: (array, array)
+    """
     D2R = pi / 180.0
     R = 6370997
     dlon = (lon - lon0) * D2R
@@ -440,6 +469,16 @@ def coordinates_to_local(lon, lat, lon0, lat0):
 
 @njit(cache=True, fastmath=True)
 def local_to_coordinates(x, y, lon0, lat0):
+    """
+    Take local coordinates (in m) to transform to latlong.
+
+    :param array x: coordinates to transform
+    :param array y: coordinates to transform
+    :param float lon0: longitude of local reference
+    :param float lat0: latitude of local reference
+    :return: lon,lat
+    :retype: (array, array)
+    """
     D2R = pi / 180.0
     R = 6370997
     d = (x ** 2 + y ** 2) ** 0.5 / R
@@ -453,3 +492,63 @@ def local_to_coordinates(x, y, lon0, lat0):
         / D2R
     )
     return lon, lat / D2R
+
+
+@njit(cache=True, fastmath=True)
+def nearest_grd_indice(x, y, x0, y0, xstep, ystep):
+    """
+    Get nearest grid indice from a position.
+
+    :param x: longitude
+    :param y: latitude
+    :param float x0: first grid longitude
+    :param float y0: first grid latitude
+    :param float xstep: step between two longitude
+    :param float ystep: step between two latitude
+    """
+    return (
+        numba_types.int32(round(((x - x0[0]) % 360.0) / xstep)),
+        numba_types.int32(round((y - y0[0]) / ystep)),
+    )
+
+
+@njit(cache=True)
+def bbox_indice_regular(vertices, x0, y0, xstep, ystep, N, circular, x_size):
+    """
+    Get bbox indice of a contour in a regular grid.
+
+    :param vertices: vertice of contour
+    :param float x0: first grid longitude
+    :param float y0: first grid latitude
+    :param float xstep: step between two longitude
+    :param float ystep: step between two latitude
+    :param int N: shift of index to enlarge window
+    :param bool circular: To know if grid is wrappable
+    :param int x_size: Number of longitude
+    """
+    lon, lat = vertices[:, 0], vertices[:, 1]
+    lon_min, lon_max = lon.min(), lon.max()
+    lat_min, lat_max = lat.min(), lat.max()
+    i_x0, i_y0 = nearest_grd_indice(lon_min, lat_min, x0, y0, xstep, ystep)
+    i_x1, i_y1 = nearest_grd_indice(lon_max, lat_max, x0, y0, xstep, ystep)
+    if circular:
+        slice_x = (i_x0 - N) % x_size, (i_x1 + N + 1) % x_size
+    else:
+        slice_x = max(i_x0 - N, 0), i_x1 + N + 1
+    slice_y = i_y0 - N, i_y1 + N + 1
+    return slice_x, slice_y
+
+
+def build_circle(x0, y0, r):
+    """
+    Build circle from center coordinates.
+
+    :param float x0: center coordinate
+    :param float y0: center coordinate
+    :param float r: radius i meter
+    :return: x,y
+    :rtype: (array,array)
+    """
+    angle = radians(linspace(0, 360, 50))
+    x_norm, y_norm = cos(angle), sin(angle)
+    return x_norm * r + x0, y_norm * r + y0
