@@ -1,68 +1,53 @@
 # -*- coding: utf-8 -*-
 """
-===========================================================================
-This file is part of py-eddy-tracker.
-
-    py-eddy-tracker is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
-
-    py-eddy-tracker is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with py-eddy-tracker.  If not, see <http://www.gnu.org/licenses/>.
-
-Copyright (c) 2014-2017 by Evan Mason and Antoine Delepoulle
-Email: emason@imedea.uib-csic.es
-===========================================================================
-
-tracking.py
-
-Version 3.0.0
-
-===========================================================================
-
+Class to manage observations gathered in track
 """
 import logging
-from numpy import (
-    empty,
-    arange,
-    where,
-    unique,
-    interp,
-    ones,
-    bool_,
-    zeros,
-    array,
-    median,
-)
 from datetime import datetime, timedelta
-from numba import njit
-from Polygon import Polygon
-from .observation import EddiesObservations
-from .. import VAR_DESCR_inv
-from ..generic import split_line, wrap_longitude, build_index
-from ..poly import polygon_overlap, create_vertice_from_2darray
 
+from numba import njit
+from numpy import (
+    arange,
+    arctan2,
+    array,
+    bool_,
+    concatenate,
+    cos,
+    degrees,
+    empty,
+    histogram,
+    interp,
+    median,
+    nan,
+    ones,
+    radians,
+    sin,
+    unique,
+    where,
+    zeros,
+)
+from Polygon import Polygon
+
+from .. import VAR_DESCR_inv, __version__
+from ..generic import build_index, cumsum_by_track, distance, split_line, wrap_longitude
+from ..poly import create_vertice_from_2darray, merge, polygon_overlap
+from .observation import EddiesObservations
 
 logger = logging.getLogger("pet")
 
 
 class TrackEddiesObservations(EddiesObservations):
-    """Class to practice Tracking on observations
-    """
+    """Class to practice Tracking on observations"""
 
-    __slots__ = ("__obs_by_track", "__first_index_of_track")
+    __slots__ = ("__obs_by_track", "__first_index_of_track", "__nb_track")
 
     ELEMENTS = [
         "lon",
         "lat",
         "radius_s",
         "radius_e",
+        "speed_area",
+        "effective_area",
         "amplitude",
         "speed_average",
         "time",
@@ -83,9 +68,92 @@ class TrackEddiesObservations(EddiesObservations):
         super().__init__(*args, **kwargs)
         self.__first_index_of_track = None
         self.__obs_by_track = None
+        self.__nb_track = None
+
+    def iter_track(self):
+        """
+        Yield track
+        """
+        for i0, nb in zip(self.index_from_track, self.nb_obs_by_track):
+            if nb == 0:
+                continue
+            yield self.index(slice(i0, i0 + nb))
+
+    @property
+    def nb_tracks(self):
+        """
+        Will count and send number of track
+        """
+        if self.__nb_track is None:
+            if len(self) == 0:
+                self.__nb_track = 0
+            else:
+                self.__nb_track = (self.nb_obs_by_track != 0).sum()
+        return self.__nb_track
+
+    def __repr__(self):
+        content = super().__repr__()
+        t0, t1 = self.period
+        period = t1 - t0 + 1
+        nb = self.nb_obs_by_track
+        nb_obs = self.observations.shape[0]
+        m = self.virtual.astype("bool")
+        nb_m = m.sum()
+        bins_t = (1, 30, 90, 180, 270, 365, 1000, 10000)
+        nb_tracks_by_t = histogram(nb, bins=bins_t)[0]
+        nb_obs_by_t = histogram(nb, bins=bins_t, weights=nb)[0]
+        pct_tracks_by_t = nb_tracks_by_t / nb_tracks_by_t.sum() * 100.0
+        pct_obs_by_t = nb_obs_by_t / nb_obs_by_t.sum() * 100.0
+        d = self.distance_to_next() / 1000.0
+        cum_d = cumsum_by_track(d, self.tracks)
+        m_last = ones(d.shape, dtype="bool")
+        m_last[-1] = False
+        m_last[self.index_from_track[1:] - 1] = False
+        content += f"""
+    | {self.nb_tracks} tracks ({
+        nb_obs / self.nb_tracks:.2f} obs/tracks, shorter {nb[nb!=0].min()} obs, longer {nb.max()} obs)
+    |   {nb_m} filled observations ({nb_m / self.nb_tracks:.2f} obs/tracks, {nb_m / nb_obs * 100:.2f} % of total)
+    |   Intepolated speed area      : {self.speed_area[m].sum() / period / 1e12:.2f} Mkm²/day
+    |   Intepolated effective area  : {self.effective_area[m].sum() / period / 1e12:.2f} Mkm²/day
+    |   Distance by day             : Mean {d[m_last].mean():.2f} , Median {median(d[m_last]):.2f} km/day
+    |   Distance by track           : Mean {cum_d[~m_last].mean():.2f} , Median {median(cum_d[~m_last]):.2f} km/track
+    ----Distribution in lifetime:
+    |   Lifetime (days  )      {self.box_display(bins_t)}
+    |   Percent of tracks         : {self.box_display(pct_tracks_by_t)}
+    |   Percent of eddies         : {self.box_display(pct_obs_by_t)}"""
+        return content
+
+    def add_distance(self):
+        """Add a field of distance (m) between to consecutive observation, 0 for the last observation of each track"""
+        if "distance_next" in self.observations.dtype.descr:
+            return self
+        new = self.add_fields(("distance_next",))
+        new["distance_next"][:1] = self.distance_to_next()
+        return new
+
+    def distance_to_next(self):
+        """
+        :return: array of distance in m, 0 when next obs if from another track
+        :rtype: array
+        """
+        d = distance(
+            self.longitude[:-1],
+            self.latitude[:-1],
+            self.longitude[1:],
+            self.latitude[1:],
+        )
+        d[self.index_from_track[1:] - 1] = 0
+        d_ = empty(d.shape[0] + 1, dtype=d.dtype)
+        d_[:-1] = d
+        d_[-1] = 0
+        return d_
 
     def filled_by_interpolation(self, mask):
         """Filled selected values by interpolation
+
+        :param array(bool) mask: True if must be filled by interpolation
+
+        .. minigallery:: py_eddy_tracker.TrackEddiesObservations.filled_by_interpolation
         """
         nb_filled = mask.sum()
         logger.info("%d obs will be filled (unobserved)", nb_filled)
@@ -102,20 +170,19 @@ class TrackEddiesObservations(EddiesObservations):
                 continue
             # to normalize longitude before interpolation
             if var == "lon":
-                lon = self.obs[var]
-                first = where(self.obs["n"] == 0)[0]
+                lon = self.lon
+                first = where(self.n == 0)[0]
                 nb_obs = empty(first.shape, dtype="u4")
                 nb_obs[:-1] = first[1:] - first[:-1]
                 nb_obs[-1] = lon.shape[0] - first[-1]
                 lon0 = (lon[first] - 180).repeat(nb_obs)
-                self.obs[var] = (lon - lon0) % 360 + lon0
+                self.lon[:] = (lon - lon0) % 360 + lon0
             self.obs[var][mask] = interp(
                 index[mask], index[~mask], self.obs[var][~mask]
             )
 
     def extract_longer_eddies(self, nb_min, nb_obs, compress_id=True):
-        """Select eddies which are longer than nb_min
-        """
+        """Select eddies which are longer than nb_min"""
         mask = nb_obs >= nb_min
         nb_obs_select = mask.sum()
         logger.info("Selection of %d observations", nb_obs_select)
@@ -126,11 +193,11 @@ class TrackEddiesObservations(EddiesObservations):
             var = field[0]
             eddies.obs[var] = self.obs[var][mask]
         if compress_id:
-            list_id = unique(eddies.obs["track"])
+            list_id = unique(eddies.obs.track)
             list_id.sort()
             id_translate = arange(list_id.max() + 1)
             id_translate[list_id] = arange(len(list_id)) + 1
-            eddies.obs["track"] = id_translate[eddies.obs["track"]]
+            eddies.track = id_translate[eddies.track]
         return eddies
 
     @property
@@ -140,12 +207,12 @@ class TrackEddiesObservations(EddiesObservations):
         return list(set(elements))
 
     def set_global_attr_netcdf(self, h_nc):
-        """Set global attr
-        """
+        """Set global attr"""
         h_nc.title = "Cyclonic" if self.sign_type == -1 else "Anticyclonic"
         h_nc.Metadata_Conventions = "Unidata Dataset Discovery v1.0"
         h_nc.comment = "Surface product; mesoscale eddies"
         h_nc.framework_used = "https://github.com/AntSimi/py-eddy-tracker"
+        h_nc.framework_version = __version__
         h_nc.standard_name_vocabulary = (
             "NetCDF Climate and Forecast (CF) Metadata Convention Standard Name Table"
         )
@@ -158,26 +225,16 @@ class TrackEddiesObservations(EddiesObservations):
         h_nc.time_coverage_start = d_start.strftime("%Y-%m-%dT00:00:00Z")
         h_nc.time_coverage_end = d_end.strftime("%Y-%m-%dT00:00:00Z")
 
-    def extract_with_area(self, area, **kwargs):
-        """
-        Extract with a bounding box
-
-        :param dict area: 4 coordinates in a dictionary to specify bounding box (lower left corner and upper right corner)
-
-        """
-        mask = (self.latitude > area["llcrnrlat"]) * (self.latitude < area["urcrnrlat"])
-        lon0 = area["llcrnrlon"]
-        lon = (self.longitude - lon0) % 360 + lon0
-        mask *= (lon > lon0) * (lon < area["urcrnrlon"])
-        return self.__extract_with_mask(mask, **kwargs)
-
     def extract_with_period(self, period, **kwargs):
         """
         Extract with a period
 
-        :param (datetime.datetime,datetime.datetime) period: two date to define period, must be specify from 1/1/1950
+        :param (int,int) period: two date to define period, must be specify from 1/1/1950
+        :param dict kwargs: look at :py:meth:`extract_with_mask`
+        :return: Return all eddy tracks which are in bounds
+        :rtype: TrackEddiesObservations
 
-        :return: same object with selected data
+        .. minigallery:: py_eddy_tracker.TrackEddiesObservations.extract_with_period
         """
         dataset_period = self.period
         p_min, p_max = period
@@ -191,15 +248,32 @@ class TrackEddiesObservations(EddiesObservations):
             mask *= self.time <= p_max
         elif p_max < 0:
             mask *= self.time <= (dataset_period[1] + p_max)
-        return self.__extract_with_mask(mask, **kwargs)
+        return self.extract_with_mask(mask, **kwargs)
 
-    @property
-    def period(self):
+    def get_azimuth(self, equatorward=False):
         """
-        Give time coverage
-        Returns: 2 date
+        Return azimuth for each tracks.
+
+        Azimuth is compute with first and last observation
+
+        :param bool equatorward: If True,  Poleward are positive and equatorward negative
+        :rtype: array
         """
-        return self.time.min(), self.time.max()
+        i0, nb = self.index_from_track, self.nb_obs_by_track
+        i0 = i0[nb != 0]
+        i1 = i0 - 1 + nb[nb != 0]
+        lat0, lon0 = self.latitude[i0], self.longitude[i0]
+        lat1, lon1 = self.latitude[i1], self.longitude[i1]
+        lat0, lon0 = radians(lat0), radians(lon0)
+        lat1, lon1 = radians(lat1), radians(lon1)
+        dlon = lon1 - lon0
+        x = cos(lat0) * sin(lat1) - sin(lat0) * cos(lat1) * cos(dlon)
+        y = sin(dlon) * cos(lat1)
+        azimuth = degrees(arctan2(y, x)) + 90
+        if equatorward:
+            south = lat0 < 0
+            azimuth[south] *= -1
+        return azimuth
 
     def get_mask_from_id(self, tracks):
         mask = zeros(self.tracks.shape, dtype=bool_)
@@ -207,6 +281,9 @@ class TrackEddiesObservations(EddiesObservations):
         return mask
 
     def compute_index(self):
+        """
+        If obs are not sorted by track, __first_index_of_track will be unusable
+        """
         if self.__first_index_of_track is None:
             s = self.tracks.max() + 1
             # Doesn't work => core dump with numba, maybe he wait i8 instead of u4
@@ -218,6 +295,33 @@ class TrackEddiesObservations(EddiesObservations):
             compute_index(self.tracks, self.__first_index_of_track, self.__obs_by_track)
             logger.debug("... OK")
 
+    @classmethod
+    def concatenate(cls, observations):
+        eddies = super().concatenate(observations)
+        last_track = 0
+        i_start = 0
+        for obs in observations:
+            nb_obs = len(obs)
+            sl = slice(i_start, i_start + nb_obs)
+            new_track = obs.track + last_track
+            eddies.track[sl] = new_track
+            last_track = new_track.max() + 1
+            i_start += nb_obs
+        return eddies
+
+    def count_by_track(self, mask):
+        """
+        Count by track
+
+        :param array[bool] mask: Mask of boolean count +1 if true
+        :return: Return count by track
+        :rtype: array
+        """
+        s = self.tracks.max() + 1
+        obs_by_track = zeros(s, "i4")
+        count_by_track(self.tracks, mask, obs_by_track)
+        return obs_by_track
+
     @property
     def index_from_track(self):
         self.compute_index()
@@ -228,9 +332,40 @@ class TrackEddiesObservations(EddiesObservations):
         self.compute_index()
         return self.__obs_by_track
 
+    @property
+    def lifetime(self):
+        """Return for each observation lifetime"""
+        return self.nb_obs_by_track.repeat(self.nb_obs_by_track)
+
+    @property
+    def age(self):
+        """Return for each observation age in %, will be [0:100]"""
+        return self.n.astype("f4") / (self.lifetime - 1) * 100.0
+
     def extract_ids(self, tracks):
         mask = self.get_mask_from_id(array(tracks))
-        return self.__extract_with_mask(mask)
+        return self.extract_with_mask(mask)
+
+    def extract_toward_direction(self, west=True, delta_lon=None):
+        """
+        Get eddy which go in same direction
+
+        :param bool west: Only eastward eddy if True return westward
+        :param None,float delta_lon: Only eddy with more than delta_lon span in longitude
+        :return: Only eastern eddy
+        :rtype: __class__
+
+        .. minigallery:: py_eddy_tracker.TrackEddiesObservations.extract_toward_direction
+        """
+        lon = self.longitude
+        i0, nb = self.index_from_track, self.nb_obs_by_track
+        i1 = i0 - 1 + nb
+        d_lon = lon[i1] - lon[i0]
+        m = d_lon < 0 if west else d_lon > 0
+        if delta_lon is not None:
+            m *= delta_lon < d_lon
+        m = m.repeat(nb)
+        return self.extract_with_mask(m)
 
     def extract_first_obs_in_box(self, res):
         data = empty(
@@ -238,11 +373,11 @@ class TrackEddiesObservations(EddiesObservations):
         )
         data["lon"] = self.longitude - self.longitude % res
         data["lat"] = self.latitude - self.latitude % res
-        data["track"] = self.obs["track"]
+        data["track"] = self.track
         _, indexs = unique(data, return_index=True)
         mask = zeros(self.obs.shape, dtype="bool")
         mask[indexs] = True
-        return self.__extract_with_mask(mask)
+        return self.extract_with_mask(mask)
 
     def extract_in_direction(self, direction, value=0):
         nb_obs = self.nb_obs_by_track
@@ -259,23 +394,37 @@ class TrackEddiesObservations(EddiesObservations):
             mask = d_lon < 0 if "W" == direction else d_lon > 0
             mask &= abs(d_lon) > value
         mask = mask.repeat(nb_obs)
-        return self.__extract_with_mask(mask)
+        return self.extract_with_mask(mask)
 
     def extract_with_length(self, bounds):
+        """
+        Return all observations in [b0:b1]
+
+        :param (int,int) bounds: length min and max of selected eddies, if use of -1 this bound is not used
+        :return: Return all eddy tracks which have length between bounds
+        :rtype: TrackEddiesObservations
+
+        .. minigallery:: py_eddy_tracker.TrackEddiesObservations.extract_with_length
+        """
+        if len(self) == 0:
+            return self.empty_dataset()
         b0, b1 = bounds
-        if b0 >= 0 and b1 >= 0:
+        if b0 >= 0 and b1 != -1:
             track_mask = (self.nb_obs_by_track >= b0) * (self.nb_obs_by_track <= b1)
-        elif b0 < 0 and b1 >= 0:
+        elif b0 == -1 and b1 >= 0:
             track_mask = self.nb_obs_by_track <= b1
-        elif b0 >= 0 and b1 < 0:
-            track_mask = self.nb_obs_by_track > b0
+        elif b0 >= 0 and b1 == -1:
+            track_mask = self.nb_obs_by_track >= b0
         else:
             logger.warning("No valid value for bounds")
             raise Exception("One bounds must be positiv")
-        return self.__extract_with_mask(track_mask.repeat(self.nb_obs_by_track))
+        return self.extract_with_mask(track_mask.repeat(self.nb_obs_by_track))
+
+    def empty_dataset(self):
+        return self.new_like(self, 0)
 
     def loess_filter(self, half_window, xfield, yfield, inplace=True):
-        track = self.obs["track"]
+        track = self.track
         x = self.obs[xfield]
         y = self.obs[yfield]
         result = track_loess_filter(half_window, x, y, track)
@@ -284,7 +433,7 @@ class TrackEddiesObservations(EddiesObservations):
             return self
 
     def median_filter(self, half_window, xfield, yfield, inplace=True):
-        track = self.obs["track"]
+        track = self.track
         x = self.obs[xfield]
         y = self.obs[yfield]
         result = track_median_filter(half_window, x, y, track)
@@ -300,7 +449,7 @@ class TrackEddiesObservations(EddiesObservations):
             loess_half_window, "time", "lat"
         )
 
-    def __extract_with_mask(
+    def extract_with_mask(
         self,
         mask,
         full_path=False,
@@ -310,15 +459,14 @@ class TrackEddiesObservations(EddiesObservations):
     ):
         """
         Extract a subset of observations
-        Args:
-            mask: mask to select observations
-            full_path: extract full path if only one part is selected
-            remove_incomplete: delete path which are not fully selected
-            compress_id: resample track number to use a little range
-            reject_virtual: if track are only virtual in selection we remove track
 
-        Returns:
-            same object with selected observations
+        :param array(bool) mask: mask to select observations
+        :param bool full_path: extract full path if only one part is selected
+        :param bool remove_incomplete: delete path which are not fully selected
+        :param bool compress_id: resample track number to use a little range
+        :param bool reject_virtual: if track are only virtual in selection we remove track
+        :return: same object with selected observations
+        :rtype: self.__class__
         """
         if full_path and remove_incomplete:
             logger.warning(
@@ -328,7 +476,7 @@ class TrackEddiesObservations(EddiesObservations):
 
         if full_path:
             if reject_virtual:
-                mask *= ~self.obs["virtual"].astype("bool")
+                mask *= ~self.virtual.astype("bool")
             tracks = unique(self.tracks[mask])
             mask = self.get_mask_from_id(tracks)
         elif remove_incomplete:
@@ -346,24 +494,108 @@ class TrackEddiesObservations(EddiesObservations):
                 var = field[0]
                 new.obs[var] = self.obs[var][mask]
             if compress_id:
-                list_id = unique(new.obs["track"])
+                list_id = unique(new.track)
                 list_id.sort()
                 id_translate = arange(list_id.max() + 1)
                 id_translate[list_id] = arange(len(list_id)) + 1
-                new.obs["track"] = id_translate[new.obs["track"]]
+                new.track = id_translate[new.track]
         return new
 
-    def plot(self, ax, ref=None, **kwargs):
+    @staticmethod
+    def re_reference_index(index, ref):
+        if isinstance(ref, slice):
+            return index + ref.start
+        else:
+            return ref[index]
+
+    def shape_polygon(self, intern=False):
+        """
+        Get polygons which enclosed each track
+
+        :param bool intern: If True use speed contour instead of effective contour
+        :rtype: list(array, array)
+        """
+        xname, yname = self.intern(intern)
+        return [merge(track[xname], track[yname]) for track in self.iter_track()]
+
+    def display_shape(self, ax, ref=None, intern=False, **kwargs):
+        """
+        This function will draw shape of each track
+
+        :param matplotlib.axes.Axes ax: ax where drawed
+        :param float,int ref: if defined all coordinates will be wrapped with ref like west boundary
+        :param bool intern: If True use speed contour instead of effective contour
+        :param dict kwargs: keyword arguments for Axes.plot
+        :return: matplotlib mappable
+        """
         if "label" in kwargs:
-            kwargs["label"] += " (%s eddies)" % (self.nb_obs_by_track != 0).sum()
-        x, y = split_line(self.longitude, self.latitude, self.tracks)
-        if ref is not None:
-            x, y = wrap_longitude(x, y, ref, cut=True)
+            kwargs["label"] = self.format_label(kwargs["label"])
+        if len(self) == 0:
+            x, y = [], []
+        else:
+            polygons = self.shape_polygon(intern)
+            x, y = list(), list()
+            for p_ in polygons:
+                x.append((nan,))
+                y.append((nan,))
+                x.append(p_[0])
+                y.append(p_[1])
+            x, y = concatenate(x), concatenate(y)
+            if ref is not None:
+                x, y = wrap_longitude(x, y, ref, cut=True)
+        return ax.plot(x, y, **kwargs)
+
+    def close_tracks(self, other, nb_obs_min=10, **kwargs):
+        """
+        Get close from another atlas.
+
+        :param self other: Atlas to compare
+        :param int nb_obs_min: Minimal number of overlap for one track
+        :param dict kwargs: keyword arguments for match function
+        :return: return other atlas reduce to common track with self
+
+        .. warning::
+            It could be a costly operation for huge dataset
+        """
+        p0, p1 = self.period
+        indexs = list()
+        for i_self, i_other, t0, t1 in self.align_on(other, bins=range(p0, p1 + 2)):
+            i, j, s = self.index(i_self).match(other.index(i_other), **kwargs)
+            indexs.append(other.re_reference_index(j, i_other))
+        indexs = concatenate(indexs)
+        tr, nb = unique(other.track[indexs], return_counts=True)
+        return other.extract_ids(tr[nb >= nb_obs_min])
+
+    def format_label(self, label):
+        t0, t1 = self.period
+        return label.format(
+            t0=t0,
+            t1=t1,
+            nb_obs=len(self),
+            nb_tracks=(self.nb_obs_by_track != 0).sum(),
+        )
+
+    def plot(self, ax, ref=None, **kwargs):
+        """
+        This function will draw path of each track
+
+        :param matplotlib.axes.Axes ax: ax where drawed
+        :param float,int ref: if defined all coordinates will be wrapped with ref like west boundary
+        :param dict kwargs: keyword arguments for Axes.plot
+        :return: matplotlib mappable
+        """
+        if "label" in kwargs:
+            kwargs["label"] = self.format_label(kwargs["label"])
+        if len(self) == 0:
+            x, y = [], []
+        else:
+            x, y = split_line(self.longitude, self.latitude, self.tracks)
+            if ref is not None:
+                x, y = wrap_longitude(x, y, ref, cut=True)
         return ax.plot(x, y, **kwargs)
 
     def split_network(self, intern=True, **kwargs):
-        """Divide each group in track
-        """
+        """Divide each group in track"""
         track_s, track_e, track_ref = build_index(self.tracks)
         ids = empty(
             len(self),
@@ -417,7 +649,15 @@ class TrackEddiesObservations(EddiesObservations):
         # )
 
     def set_tracks(self, x, y, ids, window):
-        # Will split one group in tracks
+        """
+        Will split one group in tracks
+
+        :param array x: coordinates of group
+        :param array y: coordinates of group
+        :param ndarray ids: several fields like time, group, ...
+        :param int windows: number of days where observations could missed
+        """
+
         time_index = build_index(ids["time"])
         nb = x.shape[0]
         used = zeros(nb, dtype="bool")
@@ -496,6 +736,13 @@ def compute_index(tracks, index, number):
         previous_track = track
 
 
+@njit(cache=True)
+def count_by_track(tracks, mask, number):
+    for track, test in zip(tracks, mask):
+        if test:
+            number[track] += 1
+
+
 @njit(cache=True)
 def compute_mask_from_id(tracks, first_index, number_of_obs, mask):
     for track in tracks:
@@ -550,7 +797,7 @@ def track_loess_filter(half_window, x, y, track):
 @njit(cache=True)
 def track_median_filter(half_window, x, y, track):
     """
-    Apply a loess filter on y field
+    Apply a median filter on y field
 
     :param int,float half_window: parameter of smoother
     :param array_like x: must be growing for each track but could be irregular
